##
## Copyright (C) Optumi Inc - All rights reserved.
##
## You may only use this code under license with Optumi Inc and any distribution or modification is strictly prohibited.
## To receive a copy of the licensing terms please write to contact@optumi.com or visit us at https://www.optumi.com.
##

import optumi_core as optumi
from optumi_core.exceptions import (
    NotLoggedInException,
    ServiceException,
    OptumiException,
)


import json


class ContainerRegistry:
    """This class represents a container registry, and allows for the user to log in to a private container registry before running a container.

    Attributes:
        name (str): Integration name. If left empty this will be generated by default.
        url (str): The url of the container registry

    Methods:
        rename(newName): Rename the integration
        remove(): Remove the integration
    """

    def __init__(
        self,
        name: str = "",
        url: str = None,
        username: str = None,
        password: str = None,
    ):
        """Constructor for a ContainerRegistry object used for logging in to a private container registry. This can be called with just a name to load an existing container registry integration or with full parameters to create a new container registry integration.

        Args:
            name (str, optional): Name of the integration. If this is left empty a name will be auto generated. Defaults to "".
            url (str, optional): URL of the container registry. Defaults to None.
            username (str, optional): The username used to access the registry. Defaults to None.
            password (str, optional): The password used to access the registry. Defaults to None.

        Raises:
            OptumiException: If we are unable to load an existing container registry integration or create a new one
        """
        got_url = url != None and len(username) > 0
        got_username = username != None and len(username) > 0
        got_password = password != None and len(username) > 0
        if got_url != got_username or got_url != got_password:
            raise OptumiException(
                "Need url, username, and password to specify registry"
            )

        self._name = name

        if url is None:
            # Retrieve the container registry
            # This is not the best way to do this...
            integrations = json.loads(optumi.core.get_integrations().text)[
                "integrations"
            ]
            for integration in integrations:
                if integration["name"] == name:
                    self._name = integration["name"]
                    self._url = integration["url"]
                    return
            raise OptumiException(
                "Unable to find container registry named '" + name + "'"
            )
        else:
            # Store the container registry
            info = {
                "integrationType": "container registry",
                "name": name,
                "registryService": "generic container registry",
                "url": url,
                "username": username,
                "password": password,
            }
            res = optumi.core.add_integration(name, json.dumps(info), False).text
            try:
                integration = json.loads(res)
                self._name = integration["name"]
                self._url = integration["url"]
            except json.decoder.JSONDecodeError:
                raise OptumiException(res)

    def rename(self, newName):
        """Renames the container registry integration.

        Args:
            newName (str): The new desired name for this container registry integration.
        """
        optumi.core.rename_integration(self._name, newName)
        self._name = newName

    def remove(self):
        """Removes the container registry integration."""
        ContainerRegistry.purge(self._name)

    @property
    def name(self):
        """Returns the name of this container registry integration.

        Returns:
            str: the name of this container registry integration
        """
        return self._name

    @property
    def url(self):
        """Returns the url of this container registry integration.

        Returns:
            str: the url of this container registry integration
        """
        return self._url

    @classmethod
    def purge(cls, name: str):
        """Removes a container registry integration given a name.

        Args:
            name (str): the container registry integration to remove
        """
        optumi.core.remove_integration(name)

    def __str__(self):
        return str(self._url)
