##
## Copyright (C) Optumi Inc - All rights reserved.
##
## You may only use this code under license with Optumi Inc and any distribution or modification is strictly prohibited.
## To receive a copy of the licensing terms please write to contact@optumi.com or visit us at https://www.optumi.com.
##

import optumi_core as optumi
from optumi_core.exceptions import OptumiException

import json, re


class EnvironmentVariables:
    """This class represents environment variables, and allows for the user to set them when running a program.

    Attributes:
        name (str): Integration name. If left empty this will be generated by default.
        environ (dict): A dictionary representing environment variables with the values redacted.

    Methods:
        rename(newName): Rename the integration
        remove(): Remove the integration
    """

    def __init__(self, name: str = "", environ: dict = None):
        """Constructor for an EnvironmentVariables object used for setting environment variables when running a program. This can be called with just a name to load an existing environment variables integration or with full parameters to create a new environment variables integration.

        Args:
            name (str, optional): Name of the integration. If this is left empty a name will be auto generated. Defaults to "".
            environ (dict, optional): A dictionary representing environment variables (key-value pairs).

        Raises:
            OptumiException: If we are unable to load an environment variables registry integration or create a new one
        """
        self._name = name

        if environ is None:
            # Retrieve the environment variables
            # This is not the best way to do this...
            integrations = json.loads(optumi.core.get_integrations().text)[
                "integrations"
            ]
            for integration in integrations:
                if integration["name"] == name:
                    self._name = integration["name"]
                    self._environ = EnvironmentVariables.__format_environ(
                        integration["keys"]
                    )
                    return
            raise OptumiException(
                "Unable to find environment variable named '" + name + "'"
            )
        else:
            # Make sure the env keys are not invalid
            for key in environ:
                if not bool(re.match("^[_.A-Za-z][_A-Za-z0-9]*$", key)):
                    raise OptumiException(
                        "Invalid environment variable key '" + key + "'"
                    )

            # Store the environment variables
            info = {
                "integrationType": "environment variable",
                "name": name,
                "variables": environ,
            }
            res = optumi.core.add_integration(name, json.dumps(info), False).text
            try:
                integration = json.loads(res)
                self._name = integration["name"]
                self._environ = self.__format_environ(integration["keys"])
            except json.decoder.JSONDecodeError:
                raise OptumiException(res)

    def rename(self, newName):
        """Renames the environment variables integration.

        Args:
            newName (str): The new desired name for this environment variables integration.
        """
        optumi.core.rename_integration(self._name, newName)
        self._name = newName

    def remove(self):
        """Removes the environment variables integration."""
        EnvironmentVariables.purge(self._name)

    @property
    def name(self):
        """Returns the name of this environment variables integration.

        Returns:
            str: the name of this environment variables integration
        """
        return self._name

    @property
    def environ(self):
        """Returns a dictionary representing environment variables with the values redacted.

        Returns:
            str: A dictionary representing environment variables with the values redacted.
        """
        return dict(zip(self._environ, ["**hidden**" for _ in self._environ]))

    @classmethod
    def __format_environ(cls, keys):
        return dict(zip(keys, ["**hidden**" for _ in keys]))

    @classmethod
    def purge(cls, name: str):
        """Removes an environment variables integration given a name.

        Args:
            name (str): the environment variables integration to remove
        """
        optumi.core.remove_integration(name)

    def __str__(self):
        return str(self.environ)
